#include <windows.h>
#include "driver.h"
#include "n64\registers.h"
#include "memory_extern.h"

//** globals
HINSTANCE		hVideoDriver;
HINSTANCE		hAudioDriver;

//** video function pointers 
fnVid_UnInitDLL_T				 pfnVid_UnInitDLL				 = NULL;
fnVid_ExecuteDList_T			 pfnVid_ExecuteDList			 = NULL;
fnVid_SetUcode_T				 pfnVid_SetUCode				 = NULL;
fnVid_InitVideo_T				 pfnVid_InitVideo				 = NULL;
fnVid_ReleaseVideo_T			 pfnVid_ReleaseVideo		     = NULL;
fnVid_RefreshVisual2D_T			 pfnVid_RefreshVisual2D			 = NULL;
fnVid_CreateConnectionWithCore_T pfnVid_CreateConnectionWithCore = NULL;

//** audio function pointers 
fnAud_InitDLL_T					 pfnAud_InitDLL					 = NULL;
fnAud_CreateConnectionWithCore_T pfnAud_CreateConnectionWithCore = NULL;
fnAud_InitAudio_T				 pfnAud_InitAudio				 = NULL;
fnAud_ReleaseAudio_T			 pfnAud_ReleaseAudio			 = NULL;
fnAud_ExecuteSList_T			 pfnAud_ExecuteSList			 = NULL;

void LoadVideoDriver(char *szFileName)
{
	fnVid_InitDLL_T pfnVid_InitDLL;

	hVideoDriver = LoadLibrary(szFileName); 
	if(hVideoDriver==NULL)
	{
		MessageBox(WinData.hWnd, "Unable to load Video dll!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}

	pfnVid_InitDLL      = (fnVid_InitDLL_T)      GetProcAddress(hVideoDriver, "InitDLL"); 
	pfnVid_UnInitDLL    = (fnVid_UnInitDLL_T)    GetProcAddress(hVideoDriver, "UnInitDLL");
	pfnVid_InitVideo    = (fnVid_InitVideo_T)    GetProcAddress(hVideoDriver, "InitVideo");
	pfnVid_ReleaseVideo = (fnVid_ReleaseVideo_T) GetProcAddress(hVideoDriver, "ReleaseVideo");
	pfnVid_SetUCode     = (fnVid_SetUcode_T)     GetProcAddress(hVideoDriver, "SetUcode");
	pfnVid_ExecuteDList = (fnVid_ExecuteDList_T) GetProcAddress(hVideoDriver, "ExecuteDList");
	pfnVid_RefreshVisual2D = (fnVid_RefreshVisual2D_T) GetProcAddress(hVideoDriver, "RefreshVisual2D");
	pfnVid_CreateConnectionWithCore = (fnVid_CreateConnectionWithCore_T)GetProcAddress(hVideoDriver, "CreateConnectionWithCore");

	if(pfnVid_UnInitDLL == NULL ||
	   pfnVid_ExecuteDList == NULL ||
	   pfnVid_SetUCode == NULL ||
	   pfnVid_InitVideo == NULL ||
	   pfnVid_ReleaseVideo == NULL ||
	   pfnVid_RefreshVisual2D == NULL ||
	   pfnVid_CreateConnectionWithCore == NULL)
	{
		MessageBox(WinData.hWnd, "Unable to load all symbols from Video DLL!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}

	if(pfnVid_InitDLL() != 0)
		exit(-1);
}

void ReleaseVideoDriver()
{
	pfnVid_UnInitDLL();

	FreeLibrary(hVideoDriver);

	pfnVid_UnInitDLL = NULL;
	pfnVid_ExecuteDList = NULL;
	pfnVid_SetUCode = NULL;
	pfnVid_InitVideo = NULL;
	pfnVid_ReleaseVideo = NULL;
	pfnVid_CreateConnectionWithCore = NULL;
}


/*
===============================================================================

  Function wrappers for the Video DLL.

===============================================================================
*/
HWND Vid_InitVideo(HWND hWnd, HINSTANCE hInstance)
{
	if(pfnVid_InitVideo == NULL)
	{
		MessageBox(WinData.hWnd, "NULL video dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	return pfnVid_InitVideo(hWnd, hInstance);
}

void Vid_ReleaseVideo()
{
	if(pfnVid_ReleaseVideo == NULL)
	{
		MessageBox(WinData.hWnd, "NULL video dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnVid_ReleaseVideo();
}

void Vid_CreateConnectionWithCore(tCoreConnection *CoreConn)
{
	if(pfnVid_CreateConnectionWithCore == NULL)
	{
		MessageBox(WinData.hWnd, "NULL video dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnVid_CreateConnectionWithCore(CoreConn);
}

void Vid_SetUCode(int uCode)
{
	if(pfnVid_SetUCode == NULL)
	{
		MessageBox(WinData.hWnd, "NULL video dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnVid_SetUCode(uCode);
}

void Vid_ExecuteDList(__int32 addr)
{
	if(pfnVid_ExecuteDList == NULL)
	{
		MessageBox(WinData.hWnd, "NULL video dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnVid_ExecuteDList(addr);
}

void Vid_RefreshVisual2D(void)
{
	if(pfnVid_RefreshVisual2D == NULL)
	{
		MessageBox(WinData.hWnd, "NULL video dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnVid_RefreshVisual2D();
}


//***************************************************************************************



void LoadAudioDriver(char *szFileName)
{
	fnAud_InitDLL_T pfnAud_InitDLL;

	hAudioDriver = LoadLibrary(szFileName); 
	if(hAudioDriver==NULL)
	{
		MessageBox(WinData.hWnd, "Unable to load Audio dll!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}

	pfnAud_InitDLL		= (fnAud_InitDLL_T)		 GetProcAddress(hAudioDriver, "InitDLL"); 
	pfnAud_InitAudio	= (fnAud_InitAudio_T)    GetProcAddress(hAudioDriver, "InitSound"); 
	pfnAud_ReleaseAudio	= (fnAud_ReleaseAudio_T) GetProcAddress(hAudioDriver, "ReleaseSound"); 
	pfnAud_ExecuteSList	= (fnAud_ExecuteSList_T) GetProcAddress(hAudioDriver, "ExecuteSList"); 
	pfnAud_CreateConnectionWithCore = (fnAud_CreateConnectionWithCore_T) GetProcAddress(hAudioDriver, "CreateConnectionWithCore"); 

	if(pfnAud_InitDLL == NULL ||
	   pfnAud_InitAudio == NULL ||
	   pfnAud_ReleaseAudio == NULL ||
	   pfnAud_ExecuteSList == NULL ||
	   pfnAud_CreateConnectionWithCore == NULL)
	{
		MessageBox(WinData.hWnd, "Unable to load all symbols from Sound DLL!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}

	if(pfnAud_InitDLL() != 0) exit(-1);
}


void ReleaseAudioDriver()
{
	FreeLibrary(hAudioDriver);

	pfnAud_InitDLL = NULL;
	pfnAud_InitAudio = NULL;
	pfnAud_ReleaseAudio = NULL;
	pfnAud_ExecuteSList = NULL;
	pfnAud_CreateConnectionWithCore = NULL;
}

void Aud_InitAudio(HWND hWnd, HINSTANCE hInstance)
{
	if(pfnAud_InitAudio == NULL)
	{
		MessageBox(WinData.hWnd, "NULL audio dll function (pfnAud_InitAudio) pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnAud_InitAudio(hWnd, hInstance);
}

void Aud_Configure()
{
}

void Aud_ReleaseAudio()
{
	if(Aud_ReleaseAudio == NULL)
	{
		MessageBox(DisplayWinData.hWnd, "NULL audio dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnAud_ReleaseAudio();
}

void Aud_ExecuteSList()
{
	if(pfnAud_ExecuteSList == NULL)
	{
		MessageBox(DisplayWinData.hWnd, "NULL audio dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnAud_ExecuteSList();
}

void Aud_CreateConnectionWithCore(tCoreConnection *CoreConn)
{
	if(pfnAud_CreateConnectionWithCore == NULL)
	{
		MessageBox(WinData.hWnd, "NULL audio dll function pointer!", szAppName, MB_OK|MB_ICONSTOP);
		exit(-1);
	}
	pfnAud_CreateConnectionWithCore(CoreConn);
}
