// True Reality project: interface.c

// Includes
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commctrl.h>
#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include "res/resource.h"
#include "File.h"
#include "cpu_thread.h"
#include "interface.h"


#pragma comment (lib, "comctl32.lib")


typedef struct FILEID_S FILEID_T;
struct FILEID_S
{
	char szFileName[MAX_PATH];
	int  UCode;
	FILEID_T *Next;
};

typedef struct
{
	char   Name[32]; // yeah, its 20, I know... 21 with termination
	char   *Comments; // allocated when read from .ini file
	float  SizeMb;
	float  SizeMB;
	char   Country;
} ROMINFO_T;



// Globals
HWND       hListView;
char       str[128];
char       szAppName[] = "TRWinGL";
char       szClassName[] = "True Reality Class";
char       szDisplayClassName[] = "True Reality Display Window";
WINDATA_T  WinData;
WINDATA_T  DisplayWinData;
extern char bLoadedRom;
FILEID_T   *RootFileID;
WNDPROC    RealListWndProc;

// Function Declarations
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK DisplayWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ListViewProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK OverriddenListWndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CreateMainWindow(void);
BOOL CreateListView(void);



/*
===============================================================================

  WinMain()

===============================================================================
*/
BOOL InitInterface(void)
{

	if (!CreateMainWindow())
		return FALSE;

	if (!CreateListView())
		return FALSE;

	// override it so we can grab the doubleclick notifications from the fucking thing
	RealListWndProc = (WNDPROC)SetWindowLong(hListView, GWL_WNDPROC, (LONG)OverriddenListWndProc);

		if(!CreateDisplayWindow())
		{
			MessageBox(WinData.hWnd, "unable to create display window", szAppName, MB_OK);
			return FALSE;
		}

	return TRUE;
}


void ResizeInterface(void)
{
	MoveWindow(hListView, 0, 0, WinData.Width, WinData.Height, TRUE);
}


void DestroyMainWindow(void);
void DestroyListView(void);
BOOL DestroyInterface(void)
{
	//MessageBox(NULL, "DestroyInterface called", szAppName, MB_OK);
	DestroyListView();
	DestroyMainWindow();

// FIXME!!! free the linked list

	return TRUE;
}


/*
===============================================================================

  CreateMainWindow()

===============================================================================
*/

void DestroyMainWindow(void)
{
	wsprintf(str, "%i", WinData.Width);
	WritePrivateProfileString("Interface", "Width", str, WinData.iniPath);
	wsprintf(str, "%i", WinData.Height);
	WritePrivateProfileString("Interface", "Height", str, WinData.iniPath);

	wsprintf(str, "%i", WinData.PosX);
	WritePrivateProfileString("Interface", "PosX", str, WinData.iniPath);
	wsprintf(str, "%i", WinData.PosY);
	WritePrivateProfileString("Interface", "PosY", str, WinData.iniPath);
}
BOOL CreateMainWindow(void)
{
	WNDCLASS  wndclass;
	int       x, y, lx, ly;
	RECT      Rectangle;


	ZeroMemory(&wndclass, sizeof(wndclass));
	wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wndclass.lpfnWndProc   = (WNDPROC)WndProc;
    wndclass.cbWndExtra	   = sizeof(DWORD);  // why? this is how I found it, so... *shrug*
	wndclass.hInstance     = WinData.hInstance;
    wndclass.hIcon         = LoadIcon(WinData.hInstance, "NLOGO");
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
	wndclass.lpszMenuName  = "MENU_BAR";
	wndclass.lpszClassName = szAppName;

	if (!RegisterClass(&wndclass))
	{
		MessageBox(NULL, "Window class registration failed.", szAppName, MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	// Adjust for menu and border, and center the window on the screen.
    Rectangle.left	 = 0;
    Rectangle.top	 = 0;
    Rectangle.right  = GetPrivateProfileInt("Interface", "Width", WinData.Width, WinData.iniPath);
    Rectangle.bottom = GetPrivateProfileInt("Interface", "Height", WinData.Height, WinData.iniPath);

    AdjustWindowRect (&Rectangle, WS_OVERLAPPEDWINDOW, 1);	

	lx = Rectangle.right - Rectangle.left;
	ly = Rectangle.bottom - Rectangle.top;
	x = GetPrivateProfileInt("Interface", "PosX", GetSystemMetrics(SM_CXSCREEN)/2 - lx/2, WinData.iniPath);
	if(x > GetSystemMetrics(SM_CXSCREEN) - 50)
		x = GetSystemMetrics(SM_CXSCREEN)/2 - lx/2;
	y = GetPrivateProfileInt("Interface", "PosY", GetSystemMetrics(SM_CYSCREEN)/2 - ly/2, WinData.iniPath);
	if(y > GetSystemMetrics(SM_CYSCREEN) - 50)
		y = GetSystemMetrics(SM_CYSCREEN)/2 - ly/2;

	if (!(WinData.hWnd = CreateWindow(
			szAppName,            // window class name
			szAppName,            // window caption
			WS_OVERLAPPEDWINDOW,  // window style
			x,                    // initial x position
			y,                    // initial y position
			lx,                   // initial x size
			ly,                   // initial y size
			NULL,                 // parent window handle
			NULL,                 // window menu handle
			WinData.hInstance,    // program instance handle
			NULL)))               // creation parameters
	{
		MessageBox(NULL, "Window creation failed!", szAppName, MB_OK|MB_ICONSTOP);
		return FALSE;
	}

    ShowWindow(WinData.hWnd, WinData.iCmdShow);
    UpdateWindow(WinData.hWnd);

    return TRUE;
}


/*
===============================================================================

  CreateDisplayWindow()

===============================================================================
*/
BOOL CreateDisplayWindow(void)
{
	WNDCLASS  wndclass;
	int       x, y, lx, ly;
	RECT      Rectangle;


	ZeroMemory(&wndclass, sizeof(wndclass));
	wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wndclass.lpfnWndProc   = (WNDPROC)DisplayWndProc;
    wndclass.cbWndExtra	   = sizeof(DWORD);  // why? this is how I found it, so... *shrug*
	wndclass.hInstance     = WinData.hInstance;
    wndclass.hIcon         = LoadIcon(WinData.hInstance, "NLOGO");
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szDisplayClassName;

	if (!RegisterClass(&wndclass))
	{
		MessageBox(NULL, "Window class registration failed.", szAppName, MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	// Adjust for menu and border, and center the window on the screen.
    Rectangle.left	 = 0;
    Rectangle.top	 = 0;
    Rectangle.right  = DisplayWinData.Width;
    Rectangle.bottom = DisplayWinData.Height;

    AdjustWindowRect (&Rectangle, WS_OVERLAPPEDWINDOW, 0);	

	lx = Rectangle.right - Rectangle.left;
	ly = Rectangle.bottom - Rectangle.top;
	x = GetSystemMetrics(SM_CXSCREEN)/2 - lx/2;
	y = GetSystemMetrics(SM_CYSCREEN)/2 - ly/2;

	if (!(DisplayWinData.hWnd = CreateWindow(
			szDisplayClassName,            // window class name
			szAppName,            // window caption
			WS_OVERLAPPEDWINDOW,  // window style
			x,                    // initial x position
			y,                    // initial y position
			lx,                   // initial x size
			ly,                   // initial y size
			NULL,                 // parent window handle
			NULL,                 // window menu handle
			DisplayWinData.hInstance,    // program instance handle
			NULL)))               // creation parameters
	{
		MessageBox(NULL, "Window creation failed!", szAppName, MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	// not visible
    ShowWindow(DisplayWinData.hWnd, 0);

    return TRUE;
}


/*
===============================================================================

  List View stuff

===============================================================================
*/

void DestroyListView(void)
{
	wsprintf(str, "%i", ListView_GetColumnWidth(hListView, 0));
	WritePrivateProfileString("Interface", "BrowseCol0", str, WinData.iniPath);

	wsprintf(str, "%i", ListView_GetColumnWidth(hListView, 1));
	WritePrivateProfileString("Interface", "BrowseCol1", str, WinData.iniPath);

	wsprintf(str, "%i", ListView_GetColumnWidth(hListView, 2));
	WritePrivateProfileString("Interface", "BrowseCol2", str, WinData.iniPath);

	wsprintf(str, "%i", ListView_GetColumnWidth(hListView, 3));
	WritePrivateProfileString("Interface", "BrowseCol3", str, WinData.iniPath);

	wsprintf(str, "%i", ListView_GetColumnWidth(hListView, 4));
	WritePrivateProfileString("Interface", "BrowseCol4", str, WinData.iniPath);

	wsprintf(str, "%i", ListView_GetColumnWidth(hListView, 5));
	WritePrivateProfileString("Interface", "BrowseCol5", str, WinData.iniPath);

	DestroyWindow(hListView);
}

void AddItem(char *, char *, LPARAM);
void GetRomInfo(ROMINFO_T *RomInfo, char *szFileName);

BOOL CreateListView(void)
{
    LVCOLUMN lvc; 
	int iCol;
	WIN32_FIND_DATA FindInfo;
	HANDLE hSearch;
	char szFilePath[MAX_PATH];
	char szFindString[MAX_PATH];
	char szFileName[MAX_PATH];
	int offset;
	HIMAGELIST hImgList;
	HICON hIcon;
	FILEID_T *EndPointer;
	FILEID_T *TmpFileID;
	int i;

	// Force the common controls DLL to be loaded. 
	InitCommonControls();

	if (!(hListView = CreateWindow(
			WC_LISTVIEW,          // window class name
			"",                   // window caption
			WS_CHILD | LVS_REPORT | LVS_SORTASCENDING,// window style
			0,                    // initial x position
			0,                    // initial y position
			WinData.Width,        // initial x size
			WinData.Height,       // initial y size
			WinData.hWnd,         // parent window handle
			NULL,                 // window menu handle
			WinData.hInstance,    // program instance handle
			NULL)))               // creation parameters
	{
		MessageBox(NULL, "Unable to create ListView!", szAppName, MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	ShowWindow(hListView, 1);
	//UpdateWindow(hListView);

 
    // Initialize the LVCOLUMN structure. 
    lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM; 
 
    // Add the columns. 
    for (iCol = 0; iCol < 6; iCol++) 
	{ 
        lvc.iSubItem = iCol;
		switch(iCol)
		{
			case 0:
				lvc.pszText = "Internal Name";
				lvc.cx = GetPrivateProfileInt("Interface", "BrowseCol0", 175, WinData.iniPath);;
				lvc.fmt = LVCFMT_LEFT;
				break;
			case 1:
				lvc.pszText = "Country";
				lvc.cx = GetPrivateProfileInt("Interface", "BrowseCol1", 60, WinData.iniPath);;
				lvc.fmt = LVCFMT_CENTER;
				break;
			case 2:
				lvc.pszText = "Size";
				lvc.cx = GetPrivateProfileInt("Interface", "BrowseCol2", 60, WinData.iniPath);;
				lvc.fmt = LVCFMT_CENTER;
				break;
			case 3:
				lvc.pszText = "File Name";
				lvc.cx = GetPrivateProfileInt("Interface", "BrowseCol3", 100, WinData.iniPath);;
				lvc.fmt = LVCFMT_LEFT;
				break;
			case 4:
				lvc.pszText = "Comments";
				lvc.cx = GetPrivateProfileInt("Interface", "BrowseCol4", 229, WinData.iniPath);// sixteen pixels left over for a vertical scroll bar
				lvc.fmt = LVCFMT_LEFT;
				break;
			case 5:
				lvc.pszText = "UCode";
				lvc.cx = GetPrivateProfileInt("Interface", "BrowseCol5",  45, WinData.iniPath);
				lvc.fmt = LVCFMT_CENTER;
				break;

		}

        if (ListView_InsertColumn(hListView, iCol, &lvc) == -1) 
		{
			MessageBox(WinData.hWnd, "Error creating ListView column.", szAppName, MB_OK|MB_ICONSTOP);
            return FALSE; 
		}
    } 

	hImgList = ImageList_Create(GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), ILC_MASK, 12, 0);

    hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_UNKNOWN)); 
    ImageList_AddIcon(hImgList, hIcon); 
	DeleteObject(hIcon); 
	hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_MASK)); 
    ImageList_AddIcon(hImgList, hIcon); 
    DeleteObject(hIcon); 
    hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_USA)); 
    ImageList_AddIcon(hImgList, hIcon); 
	DeleteObject(hIcon); 
	hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_MASK)); 
    ImageList_AddIcon(hImgList, hIcon); 
    DeleteObject(hIcon); 
    hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_EUR)); 
    ImageList_AddIcon(hImgList, hIcon); 
	DeleteObject(hIcon); 
	hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_MASK)); 
    ImageList_AddIcon(hImgList, hIcon); 
    DeleteObject(hIcon); 
    hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_JAP)); 
    ImageList_AddIcon(hImgList, hIcon); 
	DeleteObject(hIcon); 
	hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_MASK)); 
    ImageList_AddIcon(hImgList, hIcon); 
    DeleteObject(hIcon);
    hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_GER)); 
    ImageList_AddIcon(hImgList, hIcon); 
	DeleteObject(hIcon); 
	hIcon = LoadIcon(WinData.hInstance, MAKEINTRESOURCE(FLAG_MASK)); 
    ImageList_AddIcon(hImgList, hIcon); 
    DeleteObject(hIcon);

 
    // Assign the image lists to the list view control. 
    ListView_SetImageList(hListView, hImgList, LVSIL_SMALL); 
    

 
	for(i = 0; i < 10; i++)
	{
		wsprintf(str, "RomDir%i", i);
		GetPrivateProfileString("Interface", str, "NotFound", szFilePath, MAX_PATH, WinData.iniPath);

		if(!strcmp(szFilePath, "NotFound"))
			return TRUE;

		if(strcmp(&szFilePath[strlen(szFilePath)], "\\"))
			strcat(szFilePath, "\\");
		strcpy(szFindString, szFilePath);
		strcat(szFindString, "*.*");

		hSearch = FindFirstFile(szFindString, &FindInfo);
		if(hSearch == NULL)
			return TRUE; // everything successfull, but no files on this loop


		// Do the first one
		{
			strcpy(szFileName, FindInfo.cFileName);

			TmpFileID = (FILEID_T *)malloc(sizeof(FILEID_T));
			TmpFileID->Next = NULL;

			strcpy(TmpFileID->szFileName, szFilePath);
			strcat(TmpFileID->szFileName, szFileName);

			offset = strlen(szFileName) - 4;
			if(!stricmp(szFileName+offset, ".rom") || !stricmp(szFileName+offset, ".bin") || 
			   !stricmp(szFileName+offset, ".z64") || !stricmp(szFileName+offset, ".v64") || 
			   !stricmp(szFileName+offset, ".n64") || !stricmp(szFileName+offset, ".u64"))

			AddItem(szFilePath, szFileName, (LPARAM)&TmpFileID);

			RootFileID = TmpFileID;
			TmpFileID = NULL;

			EndPointer = (FILEID_T *)RootFileID;
		}

		// finish with the rest
		while(FindNextFile(hSearch, &FindInfo))
		{
			strcpy(szFileName, FindInfo.cFileName);

			TmpFileID = (FILEID_T *)malloc(sizeof(FILEID_T));
			TmpFileID->Next = NULL;

			strcpy(TmpFileID->szFileName, szFilePath);
			strcat(TmpFileID->szFileName, szFileName);

			offset = strlen(szFileName) - 4;
			if(!stricmp(szFileName+offset, ".rom") || !stricmp(szFileName+offset, ".bin") || 
			   !stricmp(szFileName+offset, ".z64") || !stricmp(szFileName+offset, ".v64") || 
			   !stricmp(szFileName+offset, ".n64") || !stricmp(szFileName+offset, ".u64"))

			AddItem(szFilePath, szFileName, (LPARAM)&TmpFileID->szFileName);

			// right now endpointer points the the last list before TmpDirEntry
			EndPointer->Next = (FILEID_T *)TmpFileID;
			EndPointer = TmpFileID;
			TmpFileID = NULL;

		}
	}

    return TRUE;
}


void AddItem(char *szPathName, char *szFileName, LPARAM lParam)
{
	//FILE *fp;
	FILEID_T *FileID;
	LVITEM lvi;
	LVFINDINFO lvfi;
	static int count = 0;
	int Pos;
	ROMINFO_T RomInfo;
	char str[MAX_PATH];
	char Title[32];
	char Comments[128];
	char UCode[10];
	int dummy = 0;

	FileID = (FILEID_T*)lParam;

	strcpy(str, szPathName);
	strcat(str, szFileName);
	GetRomInfo(&RomInfo, str);
	//ZeroMemory(Title, 32);

	GetPrivateProfileString(RomInfo.Name, "AltTitle", RomInfo.Name, Title, 32, WinData.iniPath);

	// Initialize and output the item entry
	ZeroMemory(&lvi, sizeof(LVITEM));
	lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM/* | LVIF_STATE*/; 
	lvi.pszText = Title;
	lvi.lParam = lParam;
	switch(RomInfo.Country)
	{
		case 0x44:// = Germany ('D')
			lvi.iImage = 8;
			break;
		case 0x45:// = USA ('E')
			lvi.iImage = 2;
			break;
		case 0x4A:// = Japan ('J')
			lvi.iImage = 6;
			break;
		case 0x50:// = Europe ('P')
			lvi.iImage = 4;
			break;
		case 0x55:// = Australia ('U')
			lvi.iImage = 0;
			break;
		default:
			lvi.iImage = 0;
			break;
	}

	ListView_InsertItem(hListView, &lvi);

	// Because we sorted alphabetically on insert, find out where it went
	lvfi.flags = LVFI_STRING;
	lvfi.psz = Title;
	Pos = ListView_FindItem(hListView, -1, &lvfi);

	// Initialize all of the entry's subcolumns
	count++;
	switch(RomInfo.Country)
	{
		case 0x0044:// = Germany ('D')
			ListView_SetItemText(hListView, Pos, 1, "Ger");
			break;
		case 0x0045:// = USA ('E')
			ListView_SetItemText(hListView, Pos, 1, "USA");
			break;
		case 0x004A:// = Japan ('J')
			ListView_SetItemText(hListView, Pos, 1, "Jap");
			break;
		case 0x0050:// = Europe ('P')
			ListView_SetItemText(hListView, Pos, 1, "Eur");
			break;
		case 0x0055:// = Australia ('U')
			ListView_SetItemText(hListView, Pos, 1, "Aus");
			break;
		default:
			wsprintf(str, "%c", RomInfo.Country);
			ListView_SetItemText(hListView, Pos, 1, str);
			break;
	}

	sprintf(str, "%3g Mbit", RomInfo.SizeMb);
	ListView_SetItemText(hListView, Pos, 2, str);
	ListView_SetItemText(hListView, Pos, 3, szFileName);

	GetPrivateProfileString(RomInfo.Name, "Comments", "", Comments, 128, WinData.iniPath);
	ListView_SetItemText(hListView, Pos, 4, Comments);

	GetPrivateProfileString(RomInfo.Name, "UCode", "todo", UCode, 10, WinData.iniPath);
	ListView_SetItemText(hListView, Pos, 5, UCode);
	sscanf(UCode, "%i",	&FileID->UCode);
}


void ByteFlip(void *in, int len)
{
	unsigned char *pointer = (unsigned char *)in;
	unsigned char tmp;
	int i;

	for(i = 0; i < len; i += 2)
	{
		tmp = pointer[i+1];
		pointer[i+1] = pointer[i];
		pointer[i] = tmp;
	}
}


void GetRomInfo(ROMINFO_T *RomInfo, char *szFileName)
{
	FILE *fp;
	char Name[32];
	short word;
	int len;
	int IsFlipped;


	fp = fopen(szFileName, "rb");
	if(fp == NULL)
	{
		strcpy(RomInfo->Name, "Error opening file!");
		return;
	}

	fseek(fp, 0, SEEK_SET);
	fread(&word, 2, 1, fp);

	if(word & 0x3780)
	{
		IsFlipped = 0;
	}
	else
	{
		if(word & 0x8037)
		{
			IsFlipped = 1;
		}
		else 
		{
			IsFlipped = -1;
		}
	}


	// size
	fseek(fp, 0, SEEK_END);
	len = ftell(fp);

	RomInfo->SizeMb = (float)len / 128.0f / 1024.0f;
	RomInfo->SizeMB = (float)len / 128.0f / 1024.0f / 8.0f;


	// internal name
	fseek(fp, 32, SEEK_SET);
	ZeroMemory(Name, 32);
	fread(Name, 20, 1, fp);
	if(IsFlipped == 1)
		ByteFlip(Name, 20);
	strcpy(RomInfo->Name, Name);


	// country code
	fseek(fp, 62, SEEK_SET);
	fread(&word, 2, 1, fp);
	if(IsFlipped == 1)
		ByteFlip(&word, 4);
	memcpy(&RomInfo->Country, &word, 1);


	fclose(fp);
}


void KeyConfigDialog(void)
{
}


/*
===============================================================================

  WndProc()

===============================================================================
*/
extern void Vid_SetUCode(int);
LRESULT CALLBACK OverriddenListWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	switch (iMsg)
	{

		case WM_RBUTTONDOWN:
			MessageBox(WinData.hWnd, "Right mouse click", szAppName, MB_OK);
			break;
		case WM_LBUTTONDBLCLK:
		{
			LVHITTESTINFO pHitInfo;
			LVITEM pItem;
			FILEID_T *FileInfo;

			pHitInfo.pt.x = LOWORD(lParam);
			pHitInfo.pt.y = HIWORD(lParam);
			ListView_HitTest(hListView, &pHitInfo);

			if(pHitInfo.iItem == -1)
				return 0;

//** Get string of the filename
			pItem.mask = LVIF_PARAM;
			pItem.iItem = pHitInfo.iItem;
			pItem.iSubItem = 0;
			ListView_GetItem(hListView, &pItem);

			FileInfo = (FILEID_T *)pItem.lParam;
			Vid_SetUCode(FileInfo->UCode);

			//MessageBox(WinData.hWnd, pFileID, szAppName, MB_OK);
			if(bLoadedRom==1)
			{
				StopCPU();
				RemoveCardridge();
			}
			StartRom(FileInfo->szFileName);
			bLoadedRom = 1;
		}
		return 0;
	}
	return CallWindowProc(RealListWndProc, hwnd, iMsg, wParam, lParam);
}








LRESULT CALLBACK ListViewProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	switch (iMsg)
	{
		case WM_CHAR:
			switch(wParam)
			{
				case VK_ESCAPE:
					SendMessage(hwnd, WM_CLOSE, 0, 0);
					break;
			}
			break;


		case WM_ERASEBKGND:
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;
	}
	return DefWindowProc (hwnd, iMsg, wParam, lParam);
}

LRESULT CALLBACK DisplayWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	switch (iMsg)
	{
		case WM_CHAR:
			switch(wParam)
			{
				case VK_ESCAPE:
					SendMessage(hwnd, WM_CLOSE, 0, 0);
					break;
			}
			break;


		case WM_ERASEBKGND:
			break;

		case WM_CLOSE:
		//case WM_DESTROY:
			StopCPU();
			ShowWindow(hwnd, 0);
			return 0;

		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;
	}
	return DefWindowProc (hwnd, iMsg, wParam, lParam);
}
